@@ -3,7 +3,7 @@ from chalk.features import Cron, Environments, Tags, description, is_primary, ow
 from chalk.features.resolver import OfflineResolver, OnlineResolver, Resolver, offline, online
 from chalk.logging import chalk_logger
 from chalk.state import State
-from chalk.streams import stream
+from chalk.streams import Windowed, stream, windowed
 from chalk.utils import AnyDataclass
 from chalk.utils.duration import CronTab, Duration, ScheduleOptions
 
@@ -22,6 +22,7 @@ __all__ = [
     "ScheduleOptions",
     "State",
     "Tags",
+    "Windowed",
     "__version__",
     "batch",
     "chalk_logger",
@@ -33,4 +34,5 @@ __all__ = [
     "realtime",
     "stream",
     "tags",
+    "windowed",
 ]
