@@ -3,6 +3,7 @@ from __future__ import annotations
 import collections.abc
 import time
 import uuid
+import warnings
 from datetime import datetime, timezone
 from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Sequence, Type, TypeVar, Union, cast
 from urllib.parse import urljoin
@@ -14,12 +15,13 @@ from requests import HTTPError
 from requests import JSONDecodeError as RequestsJSONDecodeError
 
 from chalk._version import __version__ as chalkpy_version
-from chalk.client import ChalkClient, OnlineQueryResponseProtocol
-from chalk.client.dataset import DatasetVersion, load_dataset
+from chalk.client import ChalkClient, OnlineQueryResult
+from chalk.client.dataset import DatasetImpl, DatasetVersion, dataset_from_response, load_dataset
 from chalk.client.exc import (
+    ChalkAuthException,
     ChalkBaseException,
     ChalkComputeResolverException,
-    ChalkInvalidCredentialsException,
+    ChalkDatasetDownloadException,
     ChalkOfflineQueryException,
     ChalkOnlineQueryException,
     ChalkResolverRunException,
@@ -31,6 +33,7 @@ from chalk.client.models import (
     ComputeResolverOutputResponse,
     CreateOfflineQueryJobRequest,
     CreateOfflineQueryJobResponse,
+    DatasetResponse,
     ErrorCode,
     FeatureResult,
     GetOfflineQueryJobResponse,
@@ -67,7 +70,19 @@ class _ChalkClientConfig(BaseModel):
     active_environment: Optional[str]
 
 
-class OnlineQueryResponseImpl(OnlineQueryResponseProtocol):
+def _normalize_input_times(input_times: Union[datetime, Sequence[datetime], None], input_len: int):
+    if input_times is None:
+        input_times = datetime.now(timezone.utc)
+    if isinstance(input_times, datetime):
+        input_times = [input_times for _ in range(input_len)]
+    local_tz = datetime.now(timezone.utc).astimezone().tzinfo
+
+    input_times = [x.replace(tzinfo=local_tz) if x.tzinfo is None else x for x in input_times]
+    input_times = [x.astimezone(timezone.utc) for x in input_times]
+    return input_times
+
+
+class OnlineQueryResponseImpl(OnlineQueryResult):
     data: List[FeatureResult]
     errors: List[ChalkError]
     warnings: List[str]
@@ -99,7 +114,7 @@ class OnlineQueryResponseImpl(OnlineQueryResponseProtocol):
     def __repr__(self) -> str:
         return repr(self._df_repr())
 
-    def _repr_html_(self) -> str | None:
+    def _repr_html_(self) -> Union[str, None]:
         return self._df_repr()._repr_html_()
 
     def get_feature(self, feature: Any) -> Optional[FeatureResult]:
@@ -139,7 +154,7 @@ class ChalkAPIClientImpl(ChalkClient):
             skip_cache=_skip_cache,
         )
         if token is None:
-            raise ChalkInvalidCredentialsException()
+            raise ChalkAuthException()
 
         self._config = _ChalkClientConfig(
             name=token.name or "",
@@ -154,6 +169,7 @@ class ChalkAPIClientImpl(ChalkClient):
             "Content-Type": "application/json",
             "User-Agent": f"chalkpy-{chalkpy_version}",
             "X-Chalk-Client-Id": self._config.client_id,
+            "X-Chalk-Features-Versioned": "true",
         }
         self._exchanged_credentials = False
         self._primary_environment = None
@@ -183,11 +199,13 @@ class ChalkAPIClientImpl(ChalkClient):
         self._primary_environment = creds.primary_environment
         self._exchanged_credentials = True
 
-    def _get_headers(self, environment_override: Optional[str]):
+    def _get_headers(self, environment_override: Optional[str], preview_deployment_id: Optional[str]):
         x_chalk_env_id = environment_override or self._config.active_environment or self._primary_environment
         headers = dict(self._default_headers)  # shallow copy
         if x_chalk_env_id is not None:
             headers["X-Chalk-Env-Id"] = x_chalk_env_id
+        if preview_deployment_id is not None:
+            headers["X-Chalk-Preview-Deployment"] = preview_deployment_id
         return headers
 
     @staticmethod
@@ -198,7 +216,7 @@ class ChalkAPIClientImpl(ChalkClient):
             raise exception_cls(errors=errors)
 
     @staticmethod
-    def _raise_http_error(http_error: HTTPError, exception_cls: Type[ChalkBaseException]):
+    def _raise_http_error(http_error: HTTPError, exception_cls: Type[ChalkBaseException], **exception_kwargs):
         detail = None
         try:
             response_json = http_error.response.json()
@@ -218,7 +236,7 @@ class ChalkAPIClientImpl(ChalkClient):
             code=known_error_code or ErrorCode.INTERNAL_SERVER_ERROR,
             message=detail or f"Unexpected Chalk server error with status code {status_code}",
         )
-        raise exception_cls(errors=[chalk_error])
+        raise exception_cls(errors=[chalk_error], **exception_kwargs)
 
     def _request(
         self,
@@ -229,6 +247,8 @@ class ChalkAPIClientImpl(ChalkClient):
         use_engine: bool,
         environment_override: Optional[str],
         exception_cls: Type[ChalkBaseException],
+        preview_deployment_id: Optional[str],
+        **exception_kwargs,
     ) -> T:
         # Track whether we already exchanged credentials for this request
         exchanged_credentials = False
@@ -237,8 +257,11 @@ class ChalkAPIClientImpl(ChalkClient):
             try:
                 self._exchange_credentials()
             except HTTPError as e:
-                self._raise_http_error(http_error=e, exception_cls=exception_cls)
-        headers = self._get_headers(environment_override=environment_override)
+                self._raise_http_error(http_error=e, exception_cls=exception_cls, **exception_kwargs)
+        headers = self._get_headers(
+            environment_override=environment_override,
+            preview_deployment_id=preview_deployment_id,
+        )
         url = urljoin(self._config.api_server, uri)
         json_body = json and json.dict()
         r = requests.request(method=method, headers=headers, url=url, json=json_body)
@@ -251,11 +274,9 @@ class ChalkAPIClientImpl(ChalkClient):
         try:
             r.raise_for_status()
         except HTTPError as e:
-            self._raise_http_error(http_error=e, exception_cls=exception_cls)
+            self._raise_http_error(http_error=e, exception_cls=exception_cls, **exception_kwargs)
 
-        response = response(**r.json())
-
-        return response
+        return response(**r.json())
 
     def whoami(self) -> WhoAmIResponse:
         return self._request(
@@ -266,6 +287,7 @@ class ChalkAPIClientImpl(ChalkClient):
             use_engine=False,
             environment_override=None,
             exception_cls=ChalkWhoAmIException,
+            preview_deployment_id=None,
         )
 
     def upload_features(
@@ -298,7 +320,7 @@ class ChalkAPIClientImpl(ChalkClient):
         correlation_id: Optional[str] = None,
         query_name: Optional[str] = None,
         meta: Optional[Mapping[str, str]] = None,
-    ) -> OnlineQueryResponseProtocol:
+    ) -> OnlineQueryResponseImpl:
         warnings: List[str] = []
         encoded_inputs = {}
         for feature, value in input.items():
@@ -334,9 +356,106 @@ class ChalkAPIClientImpl(ChalkClient):
             use_engine=preview_deployment_id is None,
             environment_override=context.environment if context else None,
             exception_cls=ChalkOnlineQueryException,
+            preview_deployment_id=preview_deployment_id,
         )
         return OnlineQueryResponseImpl(data=resp.data, errors=resp.errors or [], warnings=warnings)
 
+    def offline_query(
+        self,
+        input: Optional[Union[Mapping[Union[str, Feature, Any], Any], pd.DataFrame, pl.DataFrame, DataFrame]] = None,
+        input_times: Union[Sequence[datetime], datetime, None] = None,
+        output: Sequence[Union[str, Feature, Any]] = (),
+        required_output: Sequence[Union[str, Feature, Any]] = (),
+        context: Optional[OfflineQueryContext] = None,
+        dataset_name: Optional[str] = None,
+        branch: Optional[str] = None,
+        max_samples: Optional[int] = None,
+        disallow_stale_results: None = None,  # Backwards compatibility
+    ) -> DatasetImpl:
+        if disallow_stale_results is not None:
+            warnings.warn(
+                DeprecationWarning("Argument `disallow_stale_results` is deprecated. Results are always fresh.")
+            )
+        if context is None:
+            context = OfflineQueryContext()
+
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
+
+        if len(output) == 0 and len(required_output) == 0:
+            raise ValueError("Either 'output' or 'required_output' must be specified.")
+        optional_output_root_fqns = [str(f) for f in output]
+        required_output_root_fqns = [str(f) for f in required_output]
+
+        if not input:
+            # ChalkClient().sample(...)
+            response = self._create_dataset_job(
+                query_input=None,
+                optional_output=optional_output_root_fqns,
+                required_output=required_output_root_fqns,
+                max_samples=max_samples,
+                context=context,
+                dataset_name=dataset_name,
+                branch=branch,
+                disallow_stale_results=disallow_stale_results,
+            )
+            return dataset_from_response(response, self)
+
+        # ChalkClient().get_training_dataframe(...)
+        if isinstance(input, (DataFrame, pl.DataFrame)):
+            input = input.to_pandas()
+        if isinstance(input, collections.abc.Mapping):
+            input = {str(k): v for (k, v) in input.items()}
+        if not isinstance(input, pd.DataFrame):
+            input = pd.DataFrame(input)
+
+        columns = input.columns
+        matrix: List[List[Any]] = input.T.values.tolist()
+
+        columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
+
+        if input_times is None:
+            input_times = datetime.now(timezone.utc)
+        if isinstance(input_times, datetime):
+            input_times = [input_times for _ in range(len(input))]
+
+        matrix.append([a for a in input_times])
+
+        for col_index, column in enumerate(matrix):
+            for row_index, value in enumerate(column):
+                try:
+                    f = Feature.from_root_fqn(columns_fqn[col_index])
+                except FeatureNotFoundException:
+                    # The feature is not in the graph, so passing the value as-is and hoping it's possible
+                    # to json-serialize it
+                    encoded_feature = value
+                else:
+                    encoded_feature = f.converter.from_rich_to_json(
+                        value,
+                        missing_value_strategy="error",
+                    )
+
+                matrix[col_index][row_index] = encoded_feature
+
+        query_input = OfflineQueryInput(
+            columns=columns_fqn,
+            values=matrix,
+        )
+
+        response = self._create_dataset_job(
+            optional_output=optional_output_root_fqns,
+            required_output=required_output_root_fqns,
+            query_input=query_input,
+            dataset_name=dataset_name,
+            branch=branch,
+            context=context,
+            max_samples=max_samples,
+            disallow_stale_results=disallow_stale_results,
+        )
+        return dataset_from_response(response, self)
+
     def get_training_dataframe(
         self,
         input: Union[Mapping[Union[str, Feature, Any], Any], pd.DataFrame, pl.DataFrame, DataFrame],
@@ -348,8 +467,12 @@ class ChalkAPIClientImpl(ChalkClient):
         dataset: Optional[str] = None,
         branch: Optional[str] = None,
         max_samples: Optional[int] = None,
-        disallow_stale_results: bool = False,
+        disallow_stale_results: None = None,
     ) -> pd.DataFrame:
+        if disallow_stale_results is not None:
+            warnings.warn(
+                DeprecationWarning("Argument `disallow_stale_results` is deprecated. Results are always fresh.")
+            )
         if context is None:
             context = OfflineQueryContext()
         try:
@@ -379,7 +502,6 @@ class ChalkAPIClientImpl(ChalkClient):
             dataset=dataset,
             branch=branch,
             max_samples=max_samples,
-            disallow_stale_results=disallow_stale_results,
         )
 
     def _get_training_dataframe(
@@ -394,17 +516,18 @@ class ChalkAPIClientImpl(ChalkClient):
         context: OfflineQueryContext,
         dataset: Optional[str],
         branch: Optional[str],
-        disallow_stale_results: bool,
     ) -> pd.DataFrame:
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
+
         columns = input.columns
         matrix: List[List[Any]] = input.T.values.tolist()
 
         columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
 
-        if input_times is None:
-            input_times = datetime.now(timezone.utc)
-        if isinstance(input_times, datetime):
-            input_times = [input_times for _ in range(len(input))]
+        input_times = _normalize_input_times(input_times, len(input))
 
         matrix.append([a for a in input_times])
 
@@ -439,8 +562,11 @@ class ChalkAPIClientImpl(ChalkClient):
             branch=branch,
             context=context,
             max_samples=max_samples,
-            disallow_stale_results=disallow_stale_results,
+            preview_deployment_id=None,
+            lazy=False,
         )
+        if isinstance(response, pl.LazyFrame):
+            response = response.collect()
         return response.to_pandas()
 
     def sample(
@@ -453,8 +579,16 @@ class ChalkAPIClientImpl(ChalkClient):
         dataset: Optional[str] = None,
         branch: Optional[str] = None,
         context: Optional[OfflineQueryContext] = None,
-        disallow_stale_results: bool = False,
+        disallow_stale_results: None = None,
     ) -> pd.DataFrame:
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
+        if disallow_stale_results is not None:
+            warnings.warn(
+                DeprecationWarning("Argument `disallow_stale_results` is deprecated. Results are always fresh.")
+            )
         if context is None:
             context = OfflineQueryContext()
         optional_output_root_fqns = [str(f) for f in output]
@@ -473,15 +607,37 @@ class ChalkAPIClientImpl(ChalkClient):
             output_ts=output_ts,
             dataset_name=dataset,
             branch=branch,
-            disallow_stale_results=disallow_stale_results,
+            preview_deployment_id=None,
+            lazy=False,
         )
+        if isinstance(response, pl.LazyFrame):
+            response = response.collect()
 
         return response.to_pandas()
 
+    def get_dataset(
+        self,
+        dataset_name: str,
+        context: Optional[OfflineQueryContext] = None,
+        output_ts: bool = False,
+        output_id: bool = False,
+    ) -> DatasetImpl:
+        if context is None:
+            context = OfflineQueryContext()
+        response: DatasetResponse = self._get_dataset(
+            dataset_name=dataset_name,
+            context=context,
+            preview_deployment_id=None,
+        )
+        if response.errors:
+            raise ChalkDatasetDownloadException(dataset_name=dataset_name, errors=response.errors)
+        return dataset_from_response(response, self)
+
     def trigger_resolver_run(
         self,
         resolver_fqn: str,
         context: Optional[OfflineQueryContext] = None,
+        preview_deployment_id: Optional[str] = None,
     ) -> ResolverRunResponse:
         _logger.debug(f"Triggering resolver {resolver_fqn} to run")
         if context is None:
@@ -496,11 +652,17 @@ class ChalkAPIClientImpl(ChalkClient):
             use_engine=False,
             environment_override=context.environment,
             exception_cls=ChalkResolverRunException,
+            preview_deployment_id=preview_deployment_id,
         )
 
         return response
 
-    def get_run_status(self, run_id: str, context: Optional[OfflineQueryContext] = None) -> ResolverRunResponse:
+    def get_run_status(
+        self,
+        run_id: str,
+        preview_deployment_id: Optional[str] = None,
+        context: Optional[OfflineQueryContext] = None,
+    ) -> ResolverRunResponse:
         if context is None:
             context = OfflineQueryContext()
 
@@ -512,6 +674,7 @@ class ChalkAPIClientImpl(ChalkClient):
             use_engine=False,
             environment_override=context.environment,
             exception_cls=ChalkResolverRunException,
+            preview_deployment_id=preview_deployment_id,
         )
 
         return response
@@ -527,8 +690,77 @@ class ChalkAPIClientImpl(ChalkClient):
         context: OfflineQueryContext,
         output_id: bool,
         output_ts: bool,
-        disallow_stale_results: bool,
-    ) -> pl.DataFrame:
+        preview_deployment_id: Optional[str],
+        lazy: bool = True,
+    ) -> Union[pl.DataFrame, pl.LazyFrame]:
+        req = CreateOfflineQueryJobRequest(
+            output=optional_output,
+            required_output=required_output,
+            destination_format="PARQUET",
+            input=query_input,
+            max_samples=max_samples,
+            dataset_name=dataset_name,
+            branch=branch,
+        )
+        response = self._create_offline_query_job(
+            request=req,
+            context=context,
+            preview_deployment_id=None,
+        )
+        self._raise_if_200_with_errors(response=response, exception_cls=ChalkOfflineQueryException)
+        return self._await_offline_query_job(
+            job_id=response.job_id,
+            outputs=[*optional_output, *required_output],
+            lazy=lazy,
+            context=context,
+            preview_deployment_id=preview_deployment_id,
+            output_id=output_id,
+            output_ts=output_ts,
+        )
+
+    def _await_offline_query_job(
+        self,
+        job_id: uuid.UUID,
+        outputs: List[str],
+        lazy: bool,
+        context: Optional[OfflineQueryContext] = None,
+        preview_deployment_id: Optional[str] = None,
+        output_id: Optional[bool] = None,
+        output_ts: Optional[bool] = None,
+        urls_only: Optional[bool] = False,
+    ) -> Union[pl.DataFrame, pl.LazyFrame, List[str]]:
+        while True:
+            status = self._get_job_status(
+                job_id=job_id,
+                context=context,
+                preview_deployment_id=preview_deployment_id,
+            )
+            if status.is_finished:
+                break
+            time.sleep(0.5)
+        if urls_only:
+            return status.urls
+        return load_dataset(
+            uris=status.urls,
+            output_features=outputs,
+            version=DatasetVersion(status.version),
+            output_id=output_id,
+            output_ts=output_ts,
+            columns=status.columns,
+            lazy=lazy,
+        )
+
+    def _create_dataset_job(
+        self,
+        optional_output: List[str],
+        required_output: List[str],
+        query_input: Optional[OfflineQueryInput],
+        max_samples: Optional[int],
+        dataset_name: Optional[str],
+        branch: Optional[str],
+        context: OfflineQueryContext,
+        disallow_stale_results: bool = None,
+    ) -> DatasetResponse:
         req = CreateOfflineQueryJobRequest(
             output=optional_output,
             required_output=required_output,
@@ -539,21 +771,13 @@ class ChalkAPIClientImpl(ChalkClient):
             branch=branch,
             max_cache_age_secs=0 if disallow_stale_results else None,
         )
-        response = self._create_offline_query_job(request=req, context=context)
-        self._raise_if_200_with_errors(response=response, exception_cls=ChalkOfflineQueryException)
-
-        while True:
-            status = self._get_job_status(job_id=response.job_id, context=context)
-            if status.is_finished:
-                break
-            time.sleep(0.5)
-        return load_dataset(
-            uris=status.urls,
-            output_features=[*optional_output, *required_output],
-            version=DatasetVersion(response.version),
-            output_id=output_id,
-            output_ts=output_ts,
+        response = self._create_dataset_request(
+            request=req,
+            context=context,
+            preview_deployment_id=None,
         )
+        self._raise_if_200_with_errors(response=response, exception_cls=ChalkOfflineQueryException)
+        return response
 
     def compute_resolver_output(
         self,
@@ -561,7 +785,12 @@ class ChalkAPIClientImpl(ChalkClient):
         input_times: List[datetime],
         resolver: str,
         context: Optional[OfflineQueryContext] = None,
+        preview_deployment_id: Optional[str] = None,
     ) -> pl.DataFrame:
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
         if context is None:
             context = OfflineQueryContext()
         if not isinstance(input, DataFrame):
@@ -573,6 +802,8 @@ class ChalkAPIClientImpl(ChalkClient):
 
         columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
 
+        input_times = _normalize_input_times(input_times, len(input))
+
         matrix.append([a for a in input_times])
 
         for col_index, column in enumerate(matrix):
@@ -600,18 +831,36 @@ class ChalkAPIClientImpl(ChalkClient):
             environment_override=context.environment,
             use_engine=False,
             exception_cls=ChalkComputeResolverException,
+            preview_deployment_id=preview_deployment_id,
         )
         self._raise_if_200_with_errors(response=response, exception_cls=ChalkComputeResolverException)
 
         while True:
-            status = self._get_compute_job_status(job_id=response.job_id, context=context)
+            status = self._get_compute_job_status(
+                job_id=response.job_id,
+                context=context,
+                preview_deployment_id=preview_deployment_id,
+            )
             if status.is_finished:
                 break
             time.sleep(0.5)
 
-        return load_dataset(uris=status.urls, version=status.version, executor=None)
+        df = load_dataset(
+            uris=status.urls,
+            version=status.version,
+            executor=None,
+            columns=status.columns,
+        )
+        if isinstance(df, pl.LazyFrame):
+            df = df.collect()
+        return df
 
-    def _get_compute_job_status(self, job_id: str, context: OfflineQueryContext) -> GetOfflineQueryJobResponse:
+    def _get_compute_job_status(
+        self,
+        job_id: str,
+        context: OfflineQueryContext,
+        preview_deployment_id: Optional[str],
+    ) -> GetOfflineQueryJobResponse:
         return self._request(
             method="GET",
             uri=f"/v1/compute_resolver_output/{job_id}",
@@ -620,9 +869,33 @@ class ChalkAPIClientImpl(ChalkClient):
             use_engine=False,
             environment_override=context.environment,
             exception_cls=ChalkComputeResolverException,
+            preview_deployment_id=preview_deployment_id,
         )
 
-    def _create_offline_query_job(self, request: CreateOfflineQueryJobRequest, context: OfflineQueryContext):
+    def _create_dataset_request(
+        self,
+        request: CreateOfflineQueryJobRequest,
+        context: OfflineQueryContext,
+        preview_deployment_id: Optional[str],
+    ):
+        response = self._request(
+            method="POST",
+            uri="/v3/offline_query",
+            json=request,
+            response=DatasetResponse,
+            environment_override=context.environment,
+            use_engine=False,
+            exception_cls=ChalkOfflineQueryException,
+            preview_deployment_id=preview_deployment_id,
+        )
+        return response
+
+    def _create_offline_query_job(
+        self,
+        request: CreateOfflineQueryJobRequest,
+        context: OfflineQueryContext,
+        preview_deployment_id: Optional[str],
+    ):
         response = self._request(
             method="POST",
             uri="/v2/offline_query",
@@ -631,10 +904,16 @@ class ChalkAPIClientImpl(ChalkClient):
             environment_override=context.environment,
             use_engine=False,
             exception_cls=ChalkOfflineQueryException,
+            preview_deployment_id=preview_deployment_id,
         )
         return response
 
-    def _get_job_status(self, job_id: uuid.UUID, context: OfflineQueryContext) -> GetOfflineQueryJobResponse:
+    def _get_job_status(
+        self,
+        job_id: uuid.UUID,
+        context: OfflineQueryContext,
+        preview_deployment_id: Optional[str],
+    ) -> GetOfflineQueryJobResponse:
         return self._request(
             method="GET",
             uri=f"/v2/offline_query/{job_id}",
@@ -643,4 +922,41 @@ class ChalkAPIClientImpl(ChalkClient):
             use_engine=False,
             json=None,
             exception_cls=ChalkOfflineQueryException,
+            preview_deployment_id=preview_deployment_id,
+        )
+
+    def _get_dataset(
+        self,
+        dataset_name: str,
+        context: OfflineQueryContext,
+        preview_deployment_id: Optional[str],
+    ) -> DatasetResponse:
+        return self._request(
+            method="GET",
+            uri=f"/v3/offline_query/{dataset_name}",
+            response=DatasetResponse,
+            environment_override=context.environment,
+            use_engine=False,
+            json=None,
+            exception_cls=ChalkDatasetDownloadException,
+            dataset_name=dataset_name,
+            preview_deployment_id=preview_deployment_id,
+        )
+
+    def _get_dataset_urls(
+        self,
+        dataset_name: str,
+        context: OfflineQueryContext,
+        preview_deployment_id: Optional[str],
+    ) -> GetOfflineQueryJobResponse:
+        return self._request(
+            method="GET",
+            uri=f"/v2/lazy_dataset/{dataset_name}",
+            response=GetOfflineQueryJobResponse,
+            environment_override=context.environment,
+            use_engine=False,
+            json=None,
+            exception_cls=ChalkDatasetDownloadException,
+            dataset_name=dataset_name,
+            preview_deployment_id=preview_deployment_id,
         )
