@@ -48,9 +48,11 @@ class ChalkBaseException(Exception, abc.ABC):
 
 
 class ChalkWhoAmIException(ChalkBaseException):
+    """Raised from the `ChalkClient.whoami` method."""
+
     @property
     def message(self) -> str:
-        return "Failed to retrieve current user information during who-am-I check"
+        return "Failed to retrieve current user information during `whoami` check"
 
 
 class ChalkOnlineQueryException(ChalkBaseException):
@@ -70,24 +72,47 @@ class ChalkOfflineQueryException(ChalkBaseException):
 
 
 class ChalkComputeResolverException(ChalkOfflineQueryException):
+    """Exception raised when failing to compute the resolver output."""
+
     @property
     def message(self) -> str:
         return "Failed to compute resolver output"
 
 
 class ChalkResolverRunException(ChalkBaseException):
+    """Raised when failing to get the resolver's status via `ChalkClient.get_run_status`"""
+
     @property
     def message(self) -> str:
         return "Resolver run failed"
 
 
 class ChalkDatasetDownloadException(ChalkBaseException):
+    def __init__(self, dataset_name: str, errors: Optional[List[ChalkError]] = None):
+        self.dataset_name = dataset_name
+        super().__init__(errors)
+
     @property
     def message(self) -> str:
-        return "Failed to download dataset"
+        return f"Failed to download dataset '{self.dataset_name}'"
+
+    @property
+    def full_message(self) -> str:
+        return self.message + "\n" + "\n".join(["\t" + e.message for e in self.errors])
 
 
-class ChalkInvalidCredentialsException(ChalkBaseException):
+class ChalkAuthException(ChalkBaseException):
+    """Raised when constructing a `ChalkClient` without valid credentials.
+
+    When this exception is raise, no explicit `client_id` and `client_secret`
+    were provided, there was no `~/.chalk.yml` file with applicable credentials,
+    and the environment variables `CHALK_CLIENT_ID` and `CHALK_CLIENT_SECRET`
+    were not set.
+
+    You may need to run `chalk login` from your command line, or check that your
+    working directory is set to the root of your project.
+    """
+
     message = (
         "Explicit `client_id` and `client_secret` are not provided, "
         "there is no `~/.chalk.yml` file with applicable credentials, "
@@ -106,5 +131,6 @@ __all__ = [
     "ChalkResolverRunException",
     "ChalkDatasetDownloadException",
     "ChalkComputeResolverException",
+    "ChalkAuthException",
     "ChalkWhoAmIException",
 ]
