@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import uuid
 from datetime import datetime
-from enum import Enum
+from enum import Enum, IntEnum
 from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Union
 
 import numpy as np
@@ -148,7 +148,7 @@ class ErrorCodeCategory(str, Enum):
     """
 
 
-class ChalkException(BaseModel):
+class ChalkException(BaseModel, frozen=True):
     """Information about an exception from a resolver run."""
 
     kind: str
@@ -160,11 +160,8 @@ class ChalkException(BaseModel):
     stacktrace: str
     """The stacktrace produced by the code."""
 
-    class Config:
-        frozen = True
 
-
-class ChalkError(BaseModel):
+class ChalkError(BaseModel, frozen=True):
     """
     The `ChalkError` describes an error from running a resolver
     or from a feature that can't be validated.
@@ -195,9 +192,6 @@ class ChalkError(BaseModel):
     The fully qualified name of the failing resolver, e.g. `my.project.get_fraud_score`.
     """
 
-    class Config:
-        frozen = True
-
     def copy_for_feature(self, feature: str) -> "ChalkError":
         return self.copy(update={"feature": feature})
 
@@ -243,7 +237,7 @@ class WhoAmIResponse(BaseModel):
     """The ID of the user or service token making the query."""
 
 
-class FeatureResolutionMeta(BaseModel):
+class FeatureResolutionMeta(BaseModel, frozen=True):
     """Detailed metadata about the execution of an online query."""
 
     chosen_resolver_fqn: str
@@ -252,8 +246,18 @@ class FeatureResolutionMeta(BaseModel):
     cache_hit: bool
     """Whether the feature request was satisfied by a cached value."""
 
-    class Config:
-        frozen = True
+    primitive_type: Optional[str]
+    """
+    Primitive type name for the feature, e.g. `str` for `some_feature: str`.
+    Returned only if query-level 'include_meta' is True.
+    """
+
+    version: int = 1
+    """
+    The version that was selected for this feature. Defaults to `default_version`, if query
+    does not specify a constraint. If no versioning information is provided on the feature definition,
+    the default version is `1`.
+    """
 
 
 class FeatureResult(BaseModel):
@@ -306,11 +310,11 @@ class _ExchangeCredentialsResponse(BaseModel):
 
 class OfflineQueryInput(BaseModel):
     columns: List[str]
-    values: List[List[Any]]  # Values should be of TJSON
+    values: List[List[Any]]  # Values should be of type TJSON
 
 
 class OnlineQueryRequest(BaseModel):
-    inputs: Mapping[str, Any]  # Values should be of TJSON
+    inputs: Mapping[str, Any]  # Values should be of type TJSON
     outputs: List[str]
     staleness: Optional[Mapping[str, str]] = None
     context: Optional[OnlineQueryContext] = None
@@ -322,6 +326,7 @@ class OnlineQueryRequest(BaseModel):
     correlation_id: Optional[str] = None
     query_name: Optional[str] = None
     deployment_id: Optional[str] = None
+    branch_id: Optional[str] = None
     meta: Optional[Mapping[str, str]] = None
 
 
@@ -331,8 +336,41 @@ class _TriggerResolverRunRequest(BaseModel):
 
 class QueryMeta(BaseModel):
     execution_duration_s: float
+    """
+    The time, expressed in seconds, that Chalk spent executing this query.
+    """
+
     deployment_id: Optional[str] = None
+    """
+    The id of the deployment that served this query.
+    """
+
+    environment_id: Optional[str] = None
+    """
+    The id of the environment that served this query. Not intended to be human readable, but helpful for support.
+    """
+
+    environment_name: Optional[str] = None
+    """
+    The short name of the environment that served this query. For example: "dev" or "prod".
+    """
+
     query_id: Optional[str] = None
+    """
+    A unique ID generated and persisted by Chalk for this query. All computed features, metrics, and logs are
+    associated with this ID. Your system can store this ID for audit and debugging workflows.
+    """
+
+    query_timestamp: Optional[datetime] = None
+    """
+    At the start of query execution, Chalk computes 'datetime.now()'. This value is used to timestamp computed features.
+    """
+
+    query_hash: Optional[str] = None
+    """
+    Deterministic hash of the 'structure' of the query. Queries that have the same input/output features will
+    typically have the same hash; changes may be observed over time as we adjust implementation details.
+    """
 
 
 class OnlineQueryResponse(BaseModel):
@@ -386,7 +424,6 @@ class CreateOfflineQueryJobRequest(BaseModel):
     branch: Optional[str] = None
 
 
-# FIXME document
 class ComputeResolverOutputRequest(BaseModel):
     input: OfflineQueryInput
     resolver_fqn: str
@@ -437,6 +474,17 @@ class CreateOfflineQueryJobResponse(BaseModel):
     errors: Optional[List[ChalkError]] = None
 
 
+class ColumnMetadata(BaseModel):
+    feature_fqn: str = Field(description="The root FQN of the feature for a column")
+
+    column_name: str = Field(description="The name of the column that corresponds to this feature")
+
+    dtype: str = Field(description="The data type for this feature")
+    # This field is currently a JSON-stringified version of the SerializeDType property
+    # Using a string instead of a pydantic model the SerializedDType encoding does not affect
+    # the api layer
+
+
 class GetOfflineQueryJobResponse(BaseModel):
     is_finished: bool = Field(description="Whether the export job is finished (it runs asynchronously)")
     version: int = Field(
@@ -450,3 +498,68 @@ class GetOfflineQueryJobResponse(BaseModel):
         description="A list of short-lived, authenticated URLs that the client can download to retrieve the exported data."
     )
     errors: Optional[List[ChalkError]] = None
+    columns: Optional[List[ColumnMetadata]] = Field(
+        description="Expected columns for the dataframe, including data type information",
+        default=None,
+    )
+
+
+class QueryStatus(IntEnum):
+    PENDING_SUBMISSION = 1
+    """Pending submission to the database."""
+    SUBMITTED = 2
+    """Submitted to the database, but not yet running."""
+    RUNNING = 3
+    """Running in the database."""
+    ERROR = 4
+    """Error with either submitting or running the job."""
+    EXPIRED = 5
+    """The job did not complete before an expiration deadline, so there are no results."""
+    CANCELLED = 6
+    """Manually cancelled before it errored or finished successfully."""
+    SUCCESSFUL = 7  #
+    """Successfully ran the job."""
+
+
+class DatasetSampleFilter(BaseModel):
+    lower_bound: Optional[datetime] = None
+    upper_bound: Optional[datetime] = None
+    max_samples: Optional[int] = None
+
+
+class DatasetFilter(BaseModel):
+    sample_filters: DatasetSampleFilter = Field(default_factory=DatasetSampleFilter)
+    max_cache_age_secs: Optional[float] = None
+
+
+class DatasetRevisionResponse(BaseModel):
+    dataset_name: Optional[str] = None
+    dataset_id: Optional[uuid.UUID] = None
+    revision_id: uuid.UUID  # Currently, the revision ID is the job ID that created the revision
+    creator_id: str
+    outputs: List[str]
+    created_at: Optional[datetime] = None
+    started_at: Optional[datetime] = None
+    terminated_at: Optional[datetime] = None
+    givens_uri: Optional[str] = None
+    status: QueryStatus
+    filters: DatasetFilter
+    num_partitions: int
+    num_bytes: Optional[int] = None
+    output_uris: str
+    output_version: int
+
+
+class DatasetResponse(BaseModel):
+    is_finished: bool = Field(description="Whether the export job is finished (it runs asynchronously)")
+    version: int = Field(
+        default=1,  # Backwards compatibility
+        description=(
+            "Version number representing the format of the data. The client uses this version number "
+            "to properly decode and load the query results into DataFrames."
+        ),
+    )
+    dataset_id: Optional[uuid.UUID] = None
+    dataset_name: Optional[str] = None
+    revisions: List[DatasetRevisionResponse]
+    errors: Optional[List[ChalkError]] = None
