@@ -11,7 +11,6 @@ from chalk.features._encoding.missing_value import MissingValueStrategy
 from chalk.features._encoding.primitive import TPrimitive
 from chalk.features._encoding.pyarrow import pyarrow_to_polars, pyarrow_to_primitive, rich_to_pyarrow
 from chalk.features._encoding.rich import structure_primitive_to_rich, unstructure_rich_to_primitive
-from chalk.features._encoding.serialized_dtype import serialize_pyarrow_dtype
 from chalk.utils.collections import unwrap_annotated_if_needed
 from chalk.utils.json import TJSON
 from chalk.utils.log_with_context import get_logger
@@ -97,7 +96,6 @@ class FeatureConverter(Generic[_TPrim, _TRich]):
                 raise ValueError("An encoder cannot be specified without also specifying the `rich_type`")
         self._encoder = encoder
         self._decoder = decoder
-        self._serialized_dtype = serialize_pyarrow_dtype(self._pyarrow_dtype)
         self._rich_default = rich_default
         if primitive_default is ... and rich_default != ...:
             # The missing value strategy doesn't really matter because rich_default is not missing
@@ -270,10 +268,6 @@ class FeatureConverter(Generic[_TPrim, _TRich]):
     def polars_dtype(self) -> pl.PolarsDataType:
         return pyarrow_to_polars(self.pyarrow_dtype, self._name)
 
-    @property
-    def serialized_dtype(self):
-        return self._serialized_dtype
-
     @property
     def has_default(self):
         # primitive default is set <==> rich default is also set or no rich type was provided
