@@ -58,7 +58,7 @@ def features(
     max_staleness: Optional[Duration] = ...,
     name: Optional[str] = None,
 ) -> Union[Callable[[Type[T]], Type[T]], Type[T]]:
-    """Chalk lets you spell our your features directly in Python.
+    """Chalk lets you spell out your features directly in Python.
 
     Features are namespaced to a `FeatureSet`.
     To create a new `FeatureSet`, apply the `@features`
@@ -93,18 +93,22 @@ def features(
     cls
         The decorated class. You shouldn't need to pass this argument.
     name
-        The name for the feature. By default, the name of a feature is
+        The name for the feature set. By default, the name of a feature is
         taken from the name of the attribute on the class, prefixed with
-        the camel-cased name of the class. Note that if you provide an
-        explicit name, the name of the feature class will still be
-        prepended with a `.`. You may also override the name of the
+        the camel-cased name of the class.
 
     Examples
     --------
-    >>> @features(owner="andy@chalk.ai", max_staleness="30m")
+    >>> @features(
+    ...     owner="andy@chalk.ai",
+    ...     max_staleness="30m",
+    ...     etl_offline_to_online=True,
+    ...     tags="user-group",
+    ... )
     ... class User:
     ...     id: str
     ...     name: str | None
+    ...     location: LatLng
     """
 
     def wrap(c: Type[T]) -> Type[T]:
@@ -528,7 +532,7 @@ def _process_class(
         _globals = {}
 
     # Find feature times that weren't annotated.
-    for (name, member) in inspect.getmembers(cls):
+    for name, member in inspect.getmembers(cls):
         if name not in cls_annotations and isinstance(member, Windowed):
             raise TypeError(f"Windowed feature '{namespace}.{name}' is missing an annotation, like 'Windowed[str]'")
         if name not in cls_annotations and isinstance(member, Feature):
