@@ -4,6 +4,7 @@ import copy
 from collections.abc import Iterable
 from typing import Any, TypeVar, Union
 
+from chalk.features._chalkop import op
 from chalk.features.feature_field import Feature
 from chalk.features.filter import Filter
 from chalk.serialization.parsed_annotation import ParsedAnnotation
@@ -12,8 +13,6 @@ from chalk.utils.collections import ensure_tuple
 
 T = TypeVar("T")
 
-__all__ = ["FeatureWrapper", "unwrap_feature"]
-
 
 class FeatureWrapper:
     """
@@ -26,6 +25,19 @@ class FeatureWrapper:
         # Binding as a private variable as not to have naming conflicts user's use of __getattr__
         self._chalk_feature = feature
 
+    def __add__(self, other):
+        f_self = unwrap_feature(self)
+
+        if isinstance(other, FeatureWrapper):
+            f_other = unwrap_feature(other)
+            if f_other.is_scalar and f_self.is_scalar:
+                if str == f_other.converter.rich_type == f_self.converter.rich_type:
+                    return op.concat_str(self, other)
+                else:
+                    return op.sum(self, other)
+
+        raise NotImplementedError(f"Addition not implemented for '{type(other).__name__}'")
+
     def __matmul__(self, other: int):
         return FeatureWrapper(self._chalk_feature.for_version(other))
 
@@ -44,6 +56,13 @@ class FeatureWrapper:
     def __le__(self, other: object):
         return Filter(self._chalk_feature, "<=", other)
 
+    # len_registry: ClassVar[dict[int, FeatureWrapper]] = {}
+    #
+    # def __len__(self):
+    #     r = randint(0, 100_000_000_000_000)
+    #     self.len_registry[r] = self
+    #     return r
+
     def _cmp(self, op: str, other: object):
         from chalk.features.feature_field import Feature
 
@@ -129,7 +148,13 @@ class FeatureWrapper:
 
             return f_copy
 
-        raise TypeError(f"Feature {self} does not support subscripting")
+        from chalk.df.ast_parser import parse_feature_iter
+
+        if item == 0:
+            return parse_feature_iter(self)
+        elif isinstance(item, int):
+            raise StopIteration(f"Cannot subscript feature '{self}' past 0. Attempting to subscript with '{item}'")
+        raise TypeError(f"Feature '{self}' does not support subscripting. Attempted to subscript with '{item}'")
 
     def __getattr__(self, item: str):
         from chalk.features.feature_field import Feature
@@ -178,3 +203,6 @@ def ensure_feature(feature: Union[str, Feature, FeatureWrapper, Any]) -> Feature
     if isinstance(feature, Feature):
         return feature
     raise TypeError(f"Feature identifier {feature} of type {type(feature).__name__} is not supported.")
+
+
+__all__ = ["FeatureWrapper", "unwrap_feature", "ensure_feature"]
