@@ -28,13 +28,12 @@ import pyarrow
 from pydantic import BaseModel
 from typing_extensions import ParamSpec, get_args, get_origin
 
-from chalk.features.dataframe import DataFrame
+from chalk.features.dataframe import DataFrame, DataFrameMeta
 from chalk.features.feature_field import Feature
 from chalk.features.feature_set import Features
 from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
 from chalk.features.filter import Filter
 from chalk.features.tag import Environments, Tags
-from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.sink import SinkIntegrationProtocol
 from chalk.state import StateWrapper
 from chalk.streams import StreamSource, get_name_with_duration
@@ -48,7 +47,7 @@ from chalk.streams.types import (
 from chalk.utils import MachineType
 from chalk.utils.annotation_parsing import ResolverAnnotationParser
 from chalk.utils.collections import ensure_tuple
-from chalk.utils.duration import Duration, ScheduleOptions
+from chalk.utils.duration import CronTab, Duration
 from chalk.utils.environment_parsing import env_var_bool
 
 try:
@@ -76,43 +75,105 @@ class StateDescriptor(Generic[T]):
     typ: Type[T]
 
 
-class FilterFunction(Protocol):
-    def __call__(self, *args: Union[Feature, Any]) -> bool:
-        ...
-
-
-class SampleFunction(Protocol):
-    def __call__(self) -> DataFrame:
-        ...
-
-
-@dataclass
 class Cron:
     """
     Detailed options for specify the schedule and filtering
     functions for Chalk batch jobs.
     """
 
-    schedule: Optional[ScheduleOptions] = None
-    """
-    The period of the cron job. Can be either a crontab (`"0 * * * *"`)
-    or a `Duration` (`"2h"`).
-    """
+    @overload
+    def __init__(self, schedule: Union[CronTab, Duration]):
+        """Call this resolver on the provided schedule with all possible arguments.
 
-    filter: Optional[FilterFunction] = None
-    """Optionally, a function to filter down the arguments to consider.
+        Parameters
+        ----------
+        schedule
+            The period of the cron job. Can be either a crontab (`"0 * * * *"`)
+            or a `Duration` (`"2h"`).
+        """
+        ...
 
-    See https://docs.chalk.ai/docs/resolver-cron#filtering-examples for more information.
-    """
+    @overload
+    def __init__(
+        self,
+        schedule: Union[CronTab, Duration],
+        filter: Callable[[...], bool] = None,
+    ):
+        """Call this resolver on the provided schedule with all
+        possible arguments that pass the filter function.
 
-    sample: Optional[SampleFunction] = None
-    """Explicitly provide the sample function for the cron job.
+        Parameters
+        ----------
+        schedule
+            The period of the cron job. Can be either a crontab (`"0 * * * *"`)
+            or a `Duration` (`"2h"`).
+        filter
+            Optionally, a function to filter down the arguments to consider.
 
-    See https://docs.chalk.ai/docs/resolver-cron#custom-examples for more information.
-    """
+            See https://docs.chalk.ai/docs/resolver-cron#filtering-examples for more information.
+        """
+        ...
 
-    trigger_downstream: Optional[bool] = True
-    """Whether or not to trigger downstream resolvers"""
+    @overload
+    def __init__(
+        self,
+        schedule: Union[CronTab, Duration],
+        sample: Callable[[], DataFrame] = None,
+    ):
+        """Call this resolver on the provided schedule with the
+        examples provided by the sample function.
+
+        Parameters
+        ----------
+        schedule
+            The period of the cron job. Can be either a crontab (`"0 * * * *"`)
+            or a `Duration` (`"2h"`).
+        sample
+            Explicitly provide the sample function for the cron job.
+
+            See https://docs.chalk.ai/docs/resolver-cron#custom-examples for more information.
+
+        Examples
+        --------
+        >>> def s() -> DataFrame[User.id]:
+        ...     return DataFrame.read_csv(...)
+        >>> @offline(cron=Cron(schedule="1d", sample=s))
+        ... def fn(balance: User.account.balance) -> ...:
+        """
+        ...
+
+    def __init__(
+        self,
+        schedule: Union[CronTab, Duration],
+        filter: Optional[Callable[[...], bool]] = None,
+        sample: Optional[Callable[[], DataFrame]] = None,
+    ):
+        """Run an online or offline resolver on a schedule.
+
+        This class lets you add a filter or sample function
+        to your cron schedule for a resolver. See the
+        overloaded signatures for more information.
+
+        Examples
+        --------
+        Using a filter
+
+        >>> def fltr(v: Account.active):
+        ...     return v
+        >>> @online(cron=Cron(schedule="1d", filter=fltr))
+        ... def fn(balance: Account.balance) -> ...:
+
+        Using a sample function
+
+        >>> def s() -> DataFrame[User.id]:
+        ...     return DataFrame.read_csv(...)
+        >>> @offline(cron=Cron(schedule="1d", sample=s))
+        ... def fn(balance: User.account.balance) -> ...:
+        """
+        self.schedule = schedule
+        self.filter = filter
+        self.sample = sample
+        self.trigger_downstream = False
 
 
 def _flatten_features(output: Optional[Type[Features]]) -> Sequence[Feature]:
@@ -195,6 +256,8 @@ class SinkResolver(Resolver[P, T]):
         max_delay: Optional[Duration],
         upsert: Optional[bool],
         owner: Optional[str],
+        input_is_df: bool,
+        default_args: List[Optional[ResolverArgErrorHandler]],
         integration: Optional[Union[BaseSQLSourceProtocol, SinkIntegrationProtocol]] = None,
     ):
         self.owner = owner
@@ -215,17 +278,18 @@ class SinkResolver(Resolver[P, T]):
         self.max_delay = max_delay
         self.upsert = upsert
         self.integration = integration
-        self.default_args = []
+        self.default_args = default_args
         self.max_staleness = None
         self.state = None
         self.output = []
+        self.input_is_df = input_is_df
 
     def __repr__(self):
         return f"SinkResolver(name={self.fqn})"
 
 
 class OnlineResolver(Resolver[P, T]):
-    cron: Union[ScheduleOptions, Cron]
+    cron: Union[CronTab, Duration, Cron]
 
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
         return _process_call(self.fn(*args, **kwargs), declared_output=self.output)
@@ -242,7 +306,7 @@ class OnlineResolver(Resolver[P, T]):
         environment: Optional[List[str]],
         tags: Optional[List[str]],
         max_staleness: Optional[Duration],
-        cron: Optional[Union[ScheduleOptions, Cron]],
+        cron: Optional[Union[CronTab, Duration, Cron]],
         machine_type: Optional[MachineType],
         when: Optional[Filter],
         state: Optional[StateDescriptor],
@@ -291,7 +355,7 @@ class OfflineResolver(Resolver[P, T]):
         environment: Optional[List[str]],
         tags: Optional[List[str]],
         max_staleness: Optional[Duration],
-        cron: Union[ScheduleOptions, Cron],
+        cron: Union[CronTab, Duration, Cron],
         machine_type: Optional[MachineType],
         state: Optional[StateDescriptor],
         when: Optional[Filter],
@@ -336,6 +400,17 @@ class ResolverParseResult(Generic[P, T]):
     default_args: List[Optional[ResolverArgErrorHandler]]
 
 
+@dataclasses.dataclass(frozen=True)
+class SinkResolverParseResult:
+    fqn: str
+    input_features: List[Feature]
+    input_is_df: bool
+    function: Callable[P, T]
+    function_definition: str
+    doc: Optional[str]
+    input_feature_defaults: List[Optional[ResolverArgErrorHandler]]
+
+
 def get_resolver_fqn(function: Callable):
     return f"{function.__module__}.{function.__name__}"
 
@@ -378,6 +453,34 @@ def get_state_default_value(
     return default
 
 
+def _explode_windowed_features(ret_val: Type[Features]) -> Type[Features]:
+    new_features = []
+    has_windowed = False
+    is_dataframe = False
+
+    if (
+        len(ret_val.features) == 1
+        and isinstance(ret_val.features[0], type)
+        and issubclass(ret_val.features[0], DataFrame)
+    ):
+        is_dataframe = True
+
+    for f in _flatten_features(ret_val):
+        if isinstance(f, Feature) and f.is_windowed:
+            has_windowed = True
+            for d in f.window_durations:
+                windowed_name = get_name_with_duration(name_or_fqn=f.name, duration=d)
+                windowed_feature = getattr(f.features_cls, windowed_name)
+                new_features.append(unwrap_feature(windowed_feature))
+        else:
+            new_features.append(f)
+
+    if not has_windowed:
+        return ret_val
+
+    return Features[DataFrame[new_features]] if is_dataframe else Features[new_features]
+
+
 def parse_function(
     fn: Callable[P, T],
     glbs: Optional[Dict[str, Any]],
@@ -429,17 +532,28 @@ def parse_function(
     if ret_val is None and not ignore_return:
         raise ValueError(f"Resolver {fqn} did not have a valid return type")
 
+    if not is_streaming_resolver:
+        ret_val = _explode_windowed_features(ret_val)
+
     inputs = [annotation_parser.parse_annotation(p) for p in sig.parameters.keys()]
 
     # Unwrap anything that is wrapped with FeatureWrapper
     inputs = [unwrap_feature(p) if isinstance(p, FeatureWrapper) else p for p in inputs]
 
+    if len(inputs) == 0:
+        default_arg_count = 0
+    elif isinstance(inputs[0], DataFrameMeta):
+        default_arg_count = len(cast(DataFrameMeta, inputs[0]).columns)
+    else:
+        default_arg_count = len(inputs)
+
     state = None
-    default_args: List[Optional[ResolverArgErrorHandler]] = [None for _ in range(len(inputs))]
+    default_args: List[Optional[ResolverArgErrorHandler]] = [None for _ in range(default_arg_count)]
 
     for i, (arg_name, parameter) in enumerate(sig.parameters.items()):
         bad_input = lambda: ValueError(
-            f"Resolver inputs must be Features or State. Received {str(inputs[i])} for argument '{arg_name}' for '{fqn}'.\n"
+            f"Resolver inputs must be Features, DataFrame, or State. "
+            f"Received {str(inputs[i])} for argument '{arg_name}' for '{fqn}'."
         )
         arg = inputs[i]
 
@@ -459,7 +573,7 @@ def parse_function(
         if parameter.empty != parameter.default:
             default_args[i] = ResolverArgErrorHandler(parameter.default)
 
-        if not isinstance(arg, (StateWrapper, Feature)):
+        if not isinstance(arg, (StateWrapper, Feature, DataFrameMeta)):
             if allow_custom_args:
                 continue
             raise bad_input()
@@ -501,14 +615,6 @@ def parse_function(
     if not ignore_return and ret_val is not None and issubclass(ret_val, Features):
         # Streaming resolvers are themselves windowed, so the outputs must not specify a window explicitly.
         for f in _flatten_features(ret_val):
-            if f.is_windowed and not is_streaming_resolver:
-                raise TypeError(
-                    (
-                        "Windowed features need to specify a window "
-                        f"period in the return type. The feature "
-                        f"'{f.root_fqn}' returned from '{fn.__name__}' does not."
-                    )
-                )
             if f.is_windowed_pseudofeature and is_streaming_resolver:
                 feature_name_without_duration = "__".join(f.root_fqn.split("__")[:-1])  # A bit hacky, but should work
                 raise TypeError(
@@ -530,11 +636,115 @@ def parse_function(
     )
 
 
+def parse_sink_function(
+    fn: Callable[P, T],
+    glbs: Optional[Dict[str, Any]],
+    lcls: Optional[Dict[str, Any]],
+) -> SinkResolverParseResult[P, T]:
+    fqn = get_resolver_fqn(function=fn)
+    sig = inspect.signature(fn)
+    annotation_parser = ResolverAnnotationParser(fn, glbs, lcls)
+    function_definition = inspect.getsource(fn)
+    annotations = [annotation_parser.parse_annotation(p) for p in sig.parameters.keys()]
+
+    if len(annotations) == 1 and isinstance(annotations[0], DataFrameMeta):
+        # It looks like the user's function wants a DataFrame of features
+        df = cast(DataFrameMeta, annotations[0])
+        features = df.columns
+
+        return SinkResolverParseResult(
+            fqn=fqn,
+            input_is_df=True,
+            function=fn,
+            function_definition=function_definition,
+            doc=fn.__doc__,
+            input_feature_defaults=[None for _ in range(len(features))],
+            input_features=list(features),
+        )
+
+    else:
+        # It looks like the user's function wants features as individual parameters
+        feature_default_values: List[Optional[ResolverArgErrorHandler]] = []
+        feature_inputs = []
+
+        for i, (arg_name, parameter) in enumerate(sig.parameters.items()):
+            arg = annotations[i]
+            default_value = None
+            if isinstance(arg, FeatureWrapper):
+                # Unwrap anything that is wrapped with FeatureWrapper
+                arg = unwrap_feature(arg)
+
+            bad_input = lambda: ValueError(
+                f"Sink resolver inputs must be Features. Received {str(arg)} for argument '{arg_name}' for '{fqn}'.\n"
+            )
+
+            if get_origin(arg) in (UnionType, Union):  # Optional[] handling
+                args = get_args(arg)
+                if len(args) != 2:
+                    raise bad_input()
+                if type(None) not in args:
+                    raise bad_input()
+                real_arg = next((a for a in args if a is not type(None)), None)
+                if real_arg is None:
+                    raise bad_input()
+                default_value = ResolverArgErrorHandler(None)
+                arg = unwrap_feature(real_arg)
+
+            if not isinstance(arg, Feature):
+                raise bad_input()
+
+            if parameter.empty != parameter.default:
+                default_value = ResolverArgErrorHandler(parameter.default)
+
+            feature_default_values.append(default_value)
+            feature_inputs.append(arg)
+
+        return SinkResolverParseResult(
+            fqn=fqn,
+            input_features=feature_inputs,
+            function=fn,
+            function_definition=function_definition,
+            doc=fn.__doc__,
+            input_is_df=False,
+            input_feature_defaults=feature_default_values,
+        )
+
+
 class ResolverProtocol(Generic[T, P]):
+    """A resolver, returned from the decorators `@offline` and `@online`."""
+
     __is_resolver__ = True
     name: str
+    """The name of the resolver, either given by the name of the function,
+    or by the keyword argument `name` given to `@offline` or `@online`.
+    """
 
     def __call__(self, args: P) -> T:
+        """Returns the result of calling the function decorated
+        with `@offline` or `@online` with the given arguments.
+
+        Parameters
+        ----------
+        args
+            The arguments to pass to the decorated function.
+
+        Returns
+        -------
+        T
+            The result of calling the decorated function
+            with `args`
+            Useful for unit-testing.
+
+            Read more at https://docs.chalk.ai/docs/unit-tests
+
+        Examples
+        --------
+        >>> from chalk.features import online
+        >>> @online
+        ... def fn(a: int, b: int) -> int:
+        ...     return a + b
+        >>> assert fn(1, 2) == 3
+        """
         ...
 
     @classmethod
@@ -547,7 +757,7 @@ def online(
     *,
     environment: Optional[Environments] = None,
     tags: Optional[Tags] = None,
-    cron: Optional[Union[ScheduleOptions, Cron]] = None,
+    cron: Optional[Union[CronTab, Duration, Cron]] = None,
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
@@ -570,13 +780,14 @@ def online(
     *,
     environment: Optional[Environments] = None,
     tags: Optional[Tags] = None,
-    cron: Optional[Union[ScheduleOptions, Cron]] = None,
+    cron: Optional[Union[CronTab, Duration, Cron]] = None,
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
     timeout: Optional[Duration] = None,
 ) -> Union[Callable[[Callable[P, T]], ResolverProtocol[P, T]], ResolverProtocol[P, T]]:
     """Decorator to create an online resolver.
+
     Parameters
     ----------
     environment
@@ -646,9 +857,9 @@ def online(
 
     Returns
     -------
-    Callable[[Any, ...], Any]
-        A callable function! You can unit-test resolvers as you would
-        unit-test any other code.
+    Union[Callable[[Callable[P, T]], ResolverProtocol[P, T]], ResolverProtocol[P, T]]
+        A `ResolverProtocol` which can be called as a normal function! You can unit-test
+        resolvers as you would unit-test any other code.
 
         Read more at https://docs.chalk.ai/docs/unit-tests
 
@@ -711,7 +922,7 @@ def offline(
     *,
     environment: Optional[Environments] = None,
     tags: Optional[Tags] = None,
-    cron: Optional[Union[ScheduleOptions, Cron]] = None,
+    cron: Optional[Union[CronTab, Duration, Cron]] = None,
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
@@ -733,7 +944,7 @@ def offline(
     *,
     environment: Optional[Environments] = None,
     tags: Optional[Tags] = None,
-    cron: Union[ScheduleOptions, Cron] = None,
+    cron: Union[CronTab, Duration, Cron] = None,
     machine_type: Optional[MachineType] = None,
     when: Optional[Any] = None,
     owner: Optional[str] = None,
@@ -794,7 +1005,7 @@ def offline(
         is eligible to run. Unlike tags, `when` can use feature values,
         so that you can write resolvers like::
 
-        >>> @batch(when=User.risk_profile == "low" or User.is_employee)
+        >>> @offline(when=User.risk_profile == "low" or User.is_employee)
         ... def resolver_fn(...) -> ...:
         ...    ...
 
@@ -809,10 +1020,20 @@ def offline(
 
     Returns
     -------
-    Callable[[Any, ...], Any]
-        A callable function! You can unit-test resolvers
-        as you would unit test any other code.
+    Union[Callable[[Callable[P, T]], ResolverProtocol[P, T]], ResolverProtocol[P, T]]
+        A `ResolverProtocol` which can be called as a normal function! You can unit-test
+        resolvers as you would unit-test any other code.
+
         Read more at https://docs.chalk.ai/docs/unit-tests
+
+    Examples
+    --------
+    >>> @offline(cron="1h")
+    ... def get_fraud_score(
+    ...     email: User.email,
+    ...     name: User.name,
+    ... ) -> User.fraud_score:
+    ...     return socure.get_sigma_score(email, name)
     """
     caller_frame = inspect.stack()[1]
     caller_filename = caller_frame.filename
@@ -889,7 +1110,8 @@ def sink(
     integration: Optional[BaseSQLSourceProtocol] = None,
     owner: Optional[str] = None,
 ) -> Union[Callable[[Callable[P, T]], Callable[P, T]], Callable[P, T]]:
-    """Decorator to create a stream resolver.
+    """Decorator to create a sink.
+    Read more at https://docs.chalk.ai/docs/sinks
 
     Parameters
     ----------
@@ -945,8 +1167,12 @@ def sink(
     ...     email: User.email,
     ...     phone: User.phone,
     ... ):
-    ...     user_service.update(uid=uid, email=email, phone=phone)
-
+    ...     user_service.update(
+    ...         uid=uid,
+    ...         email=email,
+    ...         phone=phone
+    ...     )
+    >>> process_updates(123, "sam@chalk.ai", "555-555-5555")
 
     Returns
     -------
@@ -961,13 +1187,13 @@ def sink(
     caller_locals = caller_frame.frame.f_locals
 
     def decorator(fn: Callable[P, T], cf: str = caller_filename):
-        parsed = parse_function(fn, caller_globals, caller_locals, ignore_return=True)
+        parsed = parse_sink_function(fn, caller_globals, caller_locals)
         resolver = SinkResolver(
             filename=cf,
             function_definition=parsed.function_definition,
             fqn=parsed.fqn,
             doc=parsed.doc,
-            inputs=parsed.inputs,
+            inputs=parsed.input_features,
             fn=fn,
             environment=None if environment is None else list(ensure_tuple(environment)),
             tags=None if tags is None else list(ensure_tuple(tags)),
@@ -978,6 +1204,8 @@ def sink(
             upsert=upsert,
             integration=integration,
             owner=owner,
+            default_args=parsed.input_feature_defaults,
+            input_is_df=parsed.input_is_df,
         )
         SinkResolver.registry.append(resolver)
         return resolver
