@@ -1,4 +1,4 @@
-from chalk._monitoring.Chart import Chart, Series, Trigger, _Formula
+from chalk._monitoring.Chart import Chart, Series, Trigger
 
 # No Formula yet
 __all__ = [
