@@ -1,9 +1,9 @@
 import collections
 from collections import defaultdict
-from enum import Enum
 from typing import Dict, List, Mapping, NewType
 
 from chalk.parsed.duplicate_input_gql import (
+    GraphLogSeverity,
     ProjectSettingsGQL,
     UpdateGraphError,
     UpsertFeatureGQL,
@@ -14,12 +14,6 @@ from chalk.utils.duration import parse_chalk_duration
 from chalk.utils.string import oxford_comma_list, to_camel_case
 
 
-class GraphLogSeverity(str, Enum):
-    ERROR = "error"
-    WARNING = "warning"
-    INFO = "info"
-
-
 class _LogBuilder:
     def __init__(self):
         self._logs: List[UpdateGraphError] = []
@@ -249,28 +243,6 @@ _BUILT_IN_SUPPORTED_TYPES = {
 }
 
 
-def _validate_custom_types_have_encoders(feature: UpsertFeatureGQL, builder: _LogBuilder):
-    if feature.scalarKind is None:
-        return
-
-    # FIXME: need to validate that the underlying windowed type is supported
-    # i.e. right now Windowed[Foobarbaz] counts even if foobarbaz can't work
-    base_classes = set(feature.scalarKind.baseClasses)
-    if (
-        len(base_classes & _BUILT_IN_SUPPORTED_TYPES) == 0
-        and not feature.scalarKind.hasEncoderAndDecoder
-        and not "CHALK_WINDOWED" in feature.scalarKind.baseClasses
-    ):
-        allowed = ", ".join([z.strip("_") for z in _BUILT_IN_SUPPORTED_TYPES])
-        builder.add_error(
-            header=f'Encoder and decoder needed for feature type "{feature.scalarKind.scalarKind}"',
-            subheader=(
-                f"Custom types require an encoder and decoder to be explicitly specified. At present, the following types are automatically supported: "
-                f'{allowed}. The feature "{feature.id.fqn}" has unsupported type "{feature.scalarKind.scalarKind}".'
-            ),
-        )
-
-
 def _validate_feature_metadata(
     features: List[UpsertFeatureGQL],
     builder: _LogBuilder,
@@ -284,10 +256,10 @@ def _validate_feature_metadata(
             self.missing_metadata = missing_metadata
 
     def get_missing_metadata(feature: UpsertFeatureGQL) -> MissingMetadata:
-        res = collections.defaultdict(list)
+        res: MissingMetadata = collections.defaultdict(list)
         for setting in request_config.validation.feature.metadata:
             metadata_name = setting.name
-            severity = setting.missing.lower()
+            severity = GraphLogSeverity(setting.missing.upper())
             if hasattr(feature, metadata_name):
                 value = getattr(feature, metadata_name)
                 is_missing = value is None or (isinstance(value, list) and len(value) == 0)
@@ -302,7 +274,7 @@ def _validate_feature_metadata(
 
         missing_messages = []
         feature_str = "feature"
-        missing_metadata_str = "missing metadata"
+        missing_metadata_header_str = "missing metadata"
         max_name_len = max([len(w.feature.id.name) for w in wrapped_features])
         feature_column_width = max(max_name_len, len(feature_str)) + 1
         get_padding = lambda s: feature_column_width - len(s)
@@ -323,9 +295,9 @@ def _validate_feature_metadata(
             return None, None
 
         padding_2 = get_padding(feature_str)
-        subheader += f"      {'-' * len(feature_str)}{' ' * padding_2}  {'-' * len(missing_metadata_str)}\n"
-        subheader += f"      {feature_str}{' ' * padding_2}  {missing_metadata_str}\n"
-        subheader += f"      {'-' * len(feature_str)}{' ' * padding_2}  {'-' * len(missing_metadata_str)}\n"
+        subheader += f"      {'-' * len(feature_str)}{' ' * padding_2}  {'-' * len(missing_metadata_header_str)}\n"
+        subheader += f"      {feature_str}{' ' * padding_2}  {missing_metadata_header_str}\n"
+        subheader += f"      {'-' * len(feature_str)}{' ' * padding_2}  {'-' * len(missing_metadata_header_str)}\n"
         subheader += "\n".join(missing_messages)
 
         return header, subheader
@@ -334,12 +306,13 @@ def _validate_feature_metadata(
         if not wrapped_features:
             return
 
-        severities = [v.value for k, v in GraphLogSeverity.__dict__.items() if not k.startswith("_")]
-        for severity in severities:
-            header, subheader = get_missing_metadata_log(wrapped_features=wrapped_features, severity=severity)
+        for severity in GraphLogSeverity:
+            header, subheader = get_missing_metadata_log(
+                wrapped_features=wrapped_features, severity=GraphLogSeverity(severity)
+            )
             if header is None:
                 continue
-            builder.add_log(header=header, subheader=subheader, severity=severity)
+            builder.add_log(header=header, subheader=subheader, severity=GraphLogSeverity(severity))
 
     def get_wrapped_features(
         namespace_features: List[UpsertFeatureGQL],
@@ -383,18 +356,18 @@ def _validate_feature_metadata_config(
     ):
         return
 
-    severities = [v.value for k, v in GraphLogSeverity.__dict__.items() if not k.startswith("_")]
+    severities_lower = [e.lower() for e in GraphLogSeverity]
     first_feature = features and features[0]
 
     for missing_metadata in request_config.validation.feature.metadata:
         metadata_name = missing_metadata.name
         severity = missing_metadata.missing
 
-        severity_lower = severity.lower()
+        severity_upper = severity.upper()
         try:
-            GraphLogSeverity(severity_lower)
+            GraphLogSeverity(severity_upper)
         except ValueError:
-            severity_choices = '" or "'.join(severities)
+            severity_choices = '" or "'.join(severities_lower)
             builder.add_warning(
                 header=f'Found invalid log severity "{severity}" config for missing metadata',
                 subheader=(
@@ -451,7 +424,6 @@ def validate_graph(request: UpsertGraphGQL) -> List[UpdateGraphError]:
         _validate_joins(feature, builder)
         _validate_etl_to_online(feature, builder)
         _validate_feature_names(feature, builder)
-        _validate_custom_types_have_encoders(feature, builder)
 
     # Validate the resolvers
     fqn_to_feature = {f.id.fqn: f for f in request.features or []}
