@@ -5,7 +5,7 @@ from typing import Any, List, Type, TypeVar
 import pydantic
 from pydantic.main import BaseModel
 
-from chalk.features import DataFrame, Feature, Filter, serialize_pyarrow_dtype
+from chalk.features import DataFrame, Feature, Filter, serialize_dtype
 from chalk.features.feature_field import HasOnePathObj
 from chalk.features.resolver import Cron, OfflineResolver, OnlineResolver, SinkResolver, StreamResolver
 from chalk.parsed.duplicate_input_gql import (
@@ -19,6 +19,7 @@ from chalk.parsed.duplicate_input_gql import (
     UpsertHasOneKindGQL,
     UpsertReferencePathComponentGQL,
     UpsertResolverGQL,
+    UpsertResolverInputUnionGQL,
     UpsertResolverOutputGQL,
     UpsertScalarKindGQL,
     UpsertSinkResolverGQL,
@@ -172,18 +173,25 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
         if isinstance(cron, Cron):
             assert t.cron.schedule is not None, "`Cron` object must be constructed with a `schedule` property."
             cron = cron.schedule
+        inputs = []
+        all_inputs = []
+        for f in t.inputs:
+            if (not isinstance(f, type) or not issubclass(f, DataFrame)) and not f.is_autogenerated:
+                feature_ref = UpsertFeatureReferenceGQL(
+                    underlying=_get_feature_id(f),
+                    path=[_get_path_component(p) for p in f.path or []],
+                )
+                all_inputs.append(UpsertResolverInputUnionGQL(feature=feature_ref))
+                inputs.append(feature_ref)
+            elif isinstance(f, type) and issubclass(f, DataFrame):
+                all_inputs.append(UpsertResolverInputUnionGQL(dataframe=_convert_df(f)))
+
         return UpsertResolverGQL(
             fqn=t.fqn,
             kind="offline" if isinstance(t, OfflineResolver) else "online",
             functionDefinition=t.function_definition,
-            inputs=[
-                UpsertFeatureReferenceGQL(
-                    underlying=_get_feature_id(f),
-                    path=[_get_path_component(p) for p in f.path or []],
-                )
-                for f in t.inputs
-                if not f.is_autogenerated
-            ],
+            inputs=inputs,
+            all_inputs=all_inputs,
             output=UpsertResolverOutputGQL(
                 features=[
                     _get_feature_id(f)
@@ -273,7 +281,7 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
                     default=t.version.default,
                     versions=[f.fqn for f in t.version.reference.values()],
                 ),
-                dtype=serialize_pyarrow_dtype(t.converter.pyarrow_dtype).dict(),
+                dtype=serialize_dtype(t.converter.pyarrow_dtype),
             )
         return UpsertFeatureGQL(
             id=UpsertFeatureIdGQL(
