@@ -128,6 +128,15 @@ def SnowflakeSource(
     schema: Optional[str] = None,
     role: Optional[str] = None,
 ) -> BaseSQLSourceProtocol:
+    """Create a Snowflake data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
     return SnowflakeSourceImpl(
         name=name,
         account_identifier=account_identifier,
@@ -160,6 +169,33 @@ def PostgreSQLSource() -> SQLSourceWithTableIngestProtocol:
     ...
 
 
+@overload
+def PostgreSQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
+    """If you have only one PostgreSQL integration, there's no need to provide
+    a distinguishing name.
+
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        Name of the integration, as configured in your dashboard.
+
+    Returns
+    -------
+    SQLSourceWithTableIngestProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = PostgreSQLSource(name="RISK")
+    """
+    ...
+
+
 @overload
 def PostgreSQLSource(
     *,
@@ -209,9 +245,48 @@ def PostgreSQLSource(
     ...
 
 
+def PostgreSQLSource(
+    *,
+    host: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
+    db: Optional[str] = None,
+    user: Optional[str] = None,
+    password: Optional[str] = None,
+    name: Optional[str] = None,
+) -> TableIngestProtocol:
+    """Create a PostgreSQL data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
+    return PostgreSQLSourceImpl(host, port, db, user, password, name)
+
+
 @overload
-def PostgreSQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
-    """If you have only one PostgreSQL integration, there's no need to provide
+def MySQLSource() -> SQLSourceWithTableIngestProtocol:
+    """If you have only one MySQL connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+
+    Returns
+    -------
+    SQLSourceWithTableIngestProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> mysql = MySQLSource()
+    """
+    ...
+
+
+@overload
+def MySQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
+    """If you have only one MySQL integration, there's no need to provide
     a distinguishing name.
 
     But what happens when you have two data sources of the same kind?
@@ -231,47 +306,11 @@ def PostgreSQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
 
     Examples
     --------
-    >>> source = PostgreSQLSource(name="RISK")
+    >>> source = MySQLSource(name="RISK")
     """
     ...
 
 
-def PostgreSQLSource(
-    *,
-    host: Optional[str] = None,
-    port: Optional[Union[int, str]] = None,
-    db: Optional[str] = None,
-    user: Optional[str] = None,
-    password: Optional[str] = None,
-    name: Optional[str] = None,
-) -> TableIngestProtocol:
-    """You can also configure the integration directly using environment
-    variables on your local machine or from those added through the
-    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
-
-    Parameters
-    ----------
-    host
-        Name of host to connect to.
-    port
-        The port number to connect to at the server host.
-    db
-        The database name.
-    user
-        PostgreSQL username to connect as.
-    password
-        The password to be used if the server demands password authentication.
-    name
-        The name of the integration, as configured in your dashboard.
-
-    Returns
-    -------
-    SQLSourceWithTableIngestProtocol
-        The SQL source for use in Chalk resolvers.
-    """
-    return PostgreSQLSourceImpl(host, port, db, user, password, name)
-
-
 @overload
 def MySQLSource(
     *,
@@ -322,51 +361,6 @@ def MySQLSource(
     ...
 
 
-@overload
-def MySQLSource() -> SQLSourceWithTableIngestProtocol:
-    """If you have only one MySQL connection that you'd like
-    to add to Chalk, you do not need to specify any arguments
-    to construct the source in your code.
-
-    Returns
-    -------
-    SQLSourceWithTableIngestProtocol
-        The SQL source for use in Chalk resolvers.
-
-    Examples
-    --------
-    >>> mysql = MySQLSource()
-    """
-    ...
-
-
-@overload
-def MySQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
-    """If you have only one MySQL integration, there's no need to provide
-    a distinguishing name.
-
-    But what happens when you have two data sources of the same kind?
-    When you create a new data source from your dashboard,
-    you have an option to provide a name for the integration.
-    You can then reference this name in the code directly.
-
-    Parameters
-    ----------
-    name
-        Name of the integration, as configured in your dashboard.
-
-    Returns
-    -------
-    SQLSourceWithTableIngestProtocol
-        The SQL source for use in Chalk resolvers.
-
-    Examples
-    --------
-    >>> source = MySQLSource(name="RISK")
-    """
-    ...
-
-
 def MySQLSource(
     *,
     host: Optional[str] = None,
@@ -376,6 +370,15 @@ def MySQLSource(
     password: Optional[str] = None,
     name: Optional[str] = None,
 ) -> SQLSourceWithTableIngestProtocol:
+    """Create a MySQL data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
     return MySQLSourceImpl(host, port, db, user, password, name)
 
 
@@ -403,9 +406,68 @@ def SQLiteInMemorySource(name: Optional[str] = None) -> SQLSourceWithTableIngest
 
 
 def SQLiteFileSource(filename: Union[str, PathLike], name: Optional[str] = None) -> SQLSourceWithTableIngestProtocol:
+    """Create a SQLite source for a file.
+
+    Parameters
+    ----------
+    filename
+        The name of the file.
+    name
+        The name to use in testing
+
+    Returns
+    -------
+    SQLSourceWithTableIngestProtocol
+        The SQL source for use in Chalk resolvers.
+    """
     return SQLiteFileSourceImpl(filename, name)
 
 
+@overload
+def RedshiftSource() -> BaseSQLSourceProtocol:
+    """If you have only one Redshift connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = RedshiftSource()
+    """
+    ...
+
+
+@overload
+def RedshiftSource(*, name: str) -> BaseSQLSourceProtocol:
+    """If you have only one Redshift integration, there's no need to provide
+    a distinguishing name.
+
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        Name of the integration, as configured in your dashboard.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = RedshiftSource(name="RISK")
+    """
+    ...
+
+
 @overload
 def RedshiftSource(
     *,
@@ -451,51 +513,6 @@ def RedshiftSource(
     ...
 
 
-@overload
-def RedshiftSource(*, name: str) -> BaseSQLSourceProtocol:
-    """If you have only one Redshift integration, there's no need to provide
-    a distinguishing name.
-
-    But what happens when you have two data sources of the same kind?
-    When you create a new data source from your dashboard,
-    you have an option to provide a name for the integration.
-    You can then reference this name in the code directly.
-
-    Parameters
-    ----------
-    name
-        Name of the integration, as configured in your dashboard.
-
-    Returns
-    -------
-    BaseSQLSourceProtocol
-        The SQL source for use in Chalk resolvers.
-
-    Examples
-    --------
-    >>> source = RedshiftSource(name="RISK")
-    """
-    ...
-
-
-@overload
-def RedshiftSource() -> BaseSQLSourceProtocol:
-    """If you have only one Redshift connection that you'd like
-    to add to Chalk, you do not need to specify any arguments
-    to construct the source in your code.
-
-    Returns
-    -------
-    BaseSQLSourceProtocol
-        The SQL source for use in Chalk resolvers.
-
-    Examples
-    --------
-    >>> source = RedshiftSource()
-    """
-    ...
-
-
 def RedshiftSource(
     *,
     host: Optional[str] = None,
@@ -504,6 +521,15 @@ def RedshiftSource(
     password: Optional[str] = None,
     name: Optional[str] = None,
 ) -> BaseSQLSourceProtocol:
+    """Create a Redshift data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
     return RedshiftSourceImpl(host, db, user, password, name)
 
 
@@ -552,6 +578,50 @@ def BigQuerySource(*, name: str) -> BaseSQLSourceProtocol:
     ...
 
 
+@overload
+def BigQuerySource(
+    *,
+    project: str = None,
+    dataset: str = None,
+    location: str = None,
+    credentials_base64: Optional[str] = None,
+    credentials_path: Optional[str] = None,
+) -> BaseSQLSourceProtocol:
+    """You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Parameters
+    ----------
+    project
+        The name of the GCP project for the BigQuery instance.
+    dataset
+        The name of the BigQuery dataset.
+    location
+        The location of the BigQuery instance.
+    credentials_base64
+        The credentials to use to connect, encoded as a base64 string.
+    credentials_path
+        The path to the credentials file to use to connect.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> import os
+    >>> source = BigQuerySource(
+    ...     project=os.getenv("BIGQUERY_PROJECT"),
+    ...     dataset=os.getenv("BIGQUERY_DATASET"),
+    ...     location=os.getenv("BIGQUERY_LOCATION"),
+    ...     credentials_base64=os.getenv("BIGQUERY_CREDENTIALS_BASE64"),
+    ... )
+    """
+    ...
+
+
 def BigQuerySource(
     *,
     name: Optional[str] = None,
@@ -561,6 +631,15 @@ def BigQuerySource(
     credentials_base64: Optional[str] = None,
     credentials_path: Optional[str] = None,
 ) -> BaseSQLSourceProtocol:
+    """Create a BigQuery data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
     return BigQuerySourceImpl(
         name=name,
         project=project,
@@ -664,6 +743,15 @@ def CloudSQLSource(
     user: Optional[str] = None,
     password: Optional[str] = None,
 ) -> BaseSQLSourceProtocol:
+    """Create a CloudSQL data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
     return CloudSQLSourceImpl(
         name=name,
         instance_name=instance_name,
