@@ -39,7 +39,7 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
         self.ingested_tables: Dict[str, Any] = {}
         BaseSQLSource.__init__(self, name=name)
 
-    def engine_args(self) -> Mapping[str, Any]:
+    def engine_args(self) -> Dict[str, Any]:
         default = dict(
             pool_size=20,
             max_overflow=60,
