@@ -3,6 +3,7 @@ from __future__ import annotations
 import contextlib
 import csv
 import io
+import logging
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Mapping, Optional, Union
 
 import pyarrow as pa
@@ -10,9 +11,15 @@ import pyarrow.csv
 
 from chalk.features import FeatureConverter
 from chalk.integrations.named import load_integration_variable
-from chalk.sql._internal.sql_source import BaseSQLSource, TableIngestMixIn, validate_dtypes_for_efficient_execution
+from chalk.sql._internal.sql_source import (
+    BaseSQLSource,
+    TableIngestMixIn,
+    UnsupportedEfficientExecutionError,
+    validate_dtypes_for_efficient_execution,
+)
 from chalk.sql.finalized_query import FinalizedChalkQuery
 from chalk.sql.protocols import SQLSourceWithTableIngestProtocol
+from chalk.utils.environment_parsing import env_var_bool
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
 
@@ -80,7 +87,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         self.ingested_tables: Dict[str, Any] = {}
         BaseSQLSource.__init__(self, name=name)
 
-    def engine_args(self) -> Mapping[str, Any]:
+    def engine_args(self) -> Dict[str, Any]:
         default = dict(
             pool_size=20,
             max_overflow=60,
@@ -118,6 +125,16 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         import psycopg2.sql
         from sqlalchemy.sql import Select
 
+        if env_var_bool("CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION"):
+            raise UnsupportedEfficientExecutionError(
+                (
+                    f"The SQL statement will be executed into SQLAlchemy objects, as the environment "
+                    "variable 'CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION' is set. Unset this variable to execute "
+                    "the query directly into a PyArrow table."
+                ),
+                log_level=logging.DEBUG,
+            )
+
         if isinstance(finalized_query.query, Select):
             validate_dtypes_for_efficient_execution(
                 finalized_query.query, _supported_sqlalchemy_types_for_pa_csv_querying
@@ -132,7 +149,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         formatted_stmt = psycopg2.sql.SQL(stmt).format(
             **{k: psycopg2.sql.Literal(v) for (k, v) in named_params.items()}
         )
-        with (self.get_engine().connect() if connection is None else contextlib.nullcontext(connection)) as cnx:
+        with self.get_engine().connect() if connection is None else contextlib.nullcontext(connection) as cnx:
             dbapi = cnx.connection
             with dbapi.cursor() as cursor:
                 buffer = io.BytesIO()
