@@ -12,7 +12,7 @@ import yaml
 from pydantic import BaseModel, ValidationError, validator
 
 from chalk import OfflineResolver, OnlineResolver
-from chalk.features import DataFrame, Feature, Features
+from chalk.features import DataFrame, Feature, Features, FeatureSetBase
 from chalk.features.resolver import ResolverArgErrorHandler, StreamResolver
 from chalk.sql import TableIngestProtocol
 from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
@@ -44,6 +44,15 @@ _SOURCES = {
     "kafka": KafkaSource,
 }
 
+_DIALECTS_SQLGLOT = {
+    SnowflakeSourceImpl: "snowflake",
+    PostgreSQLSourceImpl: "postgres",
+    MySQLSourceImpl: "mysql",
+    BigQuerySourceImpl: "bigquery",
+    RedshiftSourceImpl: "redshift",
+    SQLiteFileSourceImpl: "sqlite",
+}
+
 _RESOLVER_TYPES = {
     "offline": OfflineResolver,
     "batch": OfflineResolver,
@@ -89,6 +98,7 @@ class CommentDict(BaseModel):
     message: Optional[str]
     owner: Optional[str]
     type: Optional[str]
+    timeout: Optional[str]
 
     @validator("tags", "environment", pre=True)
     @classmethod
@@ -99,7 +109,7 @@ class CommentDict(BaseModel):
             return [value]
         raise ValueError(f"Value {value} must be a string or a list of strings.")
 
-    @validator("cron", "max_staleness")
+    @validator("cron", "max_staleness", "timeout")
     @classmethod
     def validate_timedelta(cls, string):
         try:
@@ -160,6 +170,8 @@ class GlotResult:
     glot: Optional[Expression]
     args: Optional[Dict[str, str]]
     default_args: List[Optional[str]]
+    comment_dict: Optional[CommentDict]
+    docstring: Optional[str]
     errors: List[ResolverError]
 
 
@@ -196,7 +208,7 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
     path = sql_string_result.path
 
     errors: List[ResolverError] = []
-    glot_result: GlotResult = _get_sql_glot(sql_string_result.sql_string, path)
+    glot_result: GlotResult = _get_sql_glot(sql_string=sql_string_result.sql_string, path=path, sources=sources)
     if glot_result.errors:
         return ResolverResult(resolver=None, errors=glot_result.errors, db=None, fields=None, args=None)
 
@@ -211,16 +223,28 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
             inputs.append(Feature.from_root_fqn(arg))
         except Exception as e:
             errors.append(
-                ResolverError(display=f"Cannot find input feature '{arg}' in graph, {e}", path=path, parameter=arg)
+                ResolverError(
+                    display=f"The file '{path}' references an input feature '{arg}' which does not exist",
+                    path=path,
+                    parameter=arg,
+                )
             )
     outputs: List[Feature] = []
     for output in parsed.fields.values():
+        if output.endswith("*"):
+            features = FeatureSetBase.registry[parsed.namespace].features
+            for feature in features:
+                if feature.is_scalar:
+                    outputs.append(feature)
+            continue
         try:
             outputs.append(Feature.from_root_fqn(output))
         except Exception as e:
             errors.append(
                 ResolverError(
-                    display=f"Cannot find output feature '{output}' in graph, {e}", path=path, parameter=output
+                    display=f"The file '{path}' references an output feature '{output}' which does not exist",
+                    path=path,
+                    parameter=output,
                 )
             )
 
@@ -263,7 +287,7 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
     if return_one:
         output = Features[tuple(outputs)]
     else:
-        output = DataFrame[tuple(outputs)]
+        output = Features[DataFrame[tuple(outputs)]]
 
     default_args = [ResolverArgErrorHandler(default_value) for default_value in glot_result.default_args]
 
@@ -288,7 +312,7 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
             state=None,
             default_args=default_args,
             owner=parsed.comment_dict.owner,
-            timeout=None,
+            timeout=parsed.comment_dict.timeout,
         )
     except Exception as e:
         result.errors.append(
@@ -330,14 +354,16 @@ def _get_sql_string(path: str) -> SQLStringResult:
     return sql_string_result
 
 
-def _get_sql_glot(sql_string: str, path: str) -> GlotResult:
+def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource]) -> GlotResult:
     """Get sqlglot from sql string and gracefully exit if unable to"""
     try:
         import sqlglot
         import sqlglot.expressions
     except ImportError:
         raise missing_dependency_exception("chalkpy[runtime]")
-    glot_result = GlotResult(sql_string=sql_string, glot=None, args=None, default_args=[], errors=[])
+    glot_result = GlotResult(
+        sql_string=sql_string, glot=None, args=None, default_args=[], comment_dict=None, docstring=None, errors=[]
+    )
     args = {}  # sql string -> input feature string
     variables = set(re.findall("\\${.*?\\}", sql_string))
     # replace ?{variable_name} with :variable_name for sqlalchemy, and keep track of input args necessary
@@ -375,8 +401,60 @@ def _get_sql_glot(sql_string: str, path: str) -> GlotResult:
 
     glot_result.sql_string = sql_string
     glot_result.args = args
+
+    comments = ""
+    docstring = ""
+    for comment in glot_result.sql_string.splitlines():
+        if comment.strip().startswith("--"):
+            comment = comment.strip().replace("--", "")
+            if comment.strip().startswith("-"):
+                comments += f"{comment}\n"
+            else:
+                split = comment.split(":")
+                if len(split) != 2:
+                    docstring += f"{comment.strip()}\n"
+                else:
+                    comments += f"{comment}\n"
+        else:
+            break
+
     try:
-        glot_result.glot = sqlglot.parse_one(glot_result.sql_string)
+        comment_dict = yaml.safe_load(comments)
+    except Exception as e:
+        glot_result.errors.append(
+            ResolverError(
+                display=f"Comments key-values '{comments}' must be of YAML form, {e}",
+                path=path,
+                parameter=comments,
+            )
+        )
+        return glot_result
+    try:
+        comment_dict_object = CommentDict.parse_obj(comment_dict)
+    except ValidationError as e:
+        for error in e.errors():
+            glot_result.errors.append(
+                ResolverError(
+                    display=f"Could not parse comment(s) '{'.'.join(error['loc'])}': {error['msg']}",
+                    path=path,
+                    parameter=comment_dict,
+                )
+            )
+        return glot_result
+    glot_result.comment_dict = comment_dict_object
+    glot_result.docstring = docstring.strip()
+
+    source_comment = glot_result.comment_dict.source
+    source_type = None
+    if source_comment not in _SOURCES:  # actual name of source
+        for possible_source in sources:
+            if possible_source.name == source_comment:
+                source_type = _DIALECTS_SQLGLOT.get(type(possible_source))  # dialect may be unsupported
+    else:
+        if source_comment in _DIALECTS_SQLGLOT.values():  # source type, if supported
+            source_type = source_comment
+    try:
+        glot_result.glot = sqlglot.parse_one(sql=glot_result.sql_string, read=source_type)
     except Exception as e:
         glot_result.errors.append(
             ResolverError(display=f"Cannot SQL parse {glot_result.sql_string}, {e}", path=path, parameter=None)
@@ -403,50 +481,9 @@ def _parse_glot(glot_result: GlotResult, path: str, sources: Iterable[BaseSQLSou
         errors=[],
     )
 
-    # parse comments into dictionary
-    comments = ""
-    docstring = ""
-    for comment in glot_result.glot.comments:
-        if comment.strip().startswith("-"):
-            comments += f"{comment}\n"
-        else:
-            split = comment.split(":")
-            if len(split) != 2:
-                docstring += f"{comment.strip()}\n"
-            else:
-                comments += f"{comment}\n"
-    try:
-        comment_dict = yaml.safe_load(comments)
-    except Exception as e:
-        parse_result.errors.append(
-            ResolverError(
-                display=f"Comments key-values '{comments}' must be of YAML form, {e}",
-                path=path,
-                parameter=comments,
-            )
-        )
-
-    if parse_result.errors:
-        return parse_result
-
-    try:
-        comment_dict_object = CommentDict.parse_obj(comment_dict)
-    except ValidationError as e:
-        for error in e.errors():
-            parse_result.errors.append(
-                ResolverError(
-                    display=f"Could not parse comment(s) '{'.'.join(error['loc'])}': {error['msg']}",
-                    path=path,
-                    parameter=comment_dict,
-                )
-            )
-    if parse_result.errors:
-        return parse_result
-
-    parse_result.comment_dict = comment_dict_object
-    parse_result.docstring = docstring.strip()
-
     # define a source SQL database. Can either specify name or kind if only one of the kind is present.
+    parse_result.comment_dict = glot_result.comment_dict
+    parse_result.docstring = glot_result.docstring
     source_name = parse_result.comment_dict.source
     source = None
     if source_name not in _SOURCES:  # actual name of source
@@ -477,7 +514,6 @@ def _parse_glot(glot_result: GlotResult, path: str, sources: Iterable[BaseSQLSou
 
     # get resolver fields: which columns selected will match to which chalk feature?
     namespace = parse_result.comment_dict.resolves
-    # TODO: should handle parsing of 'resolves' list of features instead of namespace in the future
     parse_result.namespace = namespace
     fields = {}  # sql string -> output feature string
 
