@@ -2,8 +2,9 @@ from __future__ import annotations
 
 import warnings
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Mapping, Optional, Sequence, Union
 
+from chalk.clogging import chalk_logger
 from chalk.features import DataFrame, Feature, FeatureConverter, Features, FeatureWrapper, ensure_feature
 from chalk.sql._internal.incremental import IncrementalSettings
 from chalk.sql.protocols import BaseSQLSourceProtocol
@@ -91,7 +92,7 @@ class FinalizedChalkQuery:
         expected_features: Optional[Sequence[Feature]] = None,
         connection: Optional[Any] = None,
     ) -> Union[Features, DataFrame, None]:
-        """Actually execute the query.
+        """Actually execute the query to a `DataFrame` or set of features.
 
         If the finalizer was ONE, ONE_OR_NONE, or FIRST, then a `Features` instance
         is returned. Otherwise, if the `finalizer` is ALL, then a `DataFrame` instance
@@ -126,8 +127,8 @@ class FinalizedChalkQuery:
         expected_features: Optional[Sequence[Feature]] = None,
         connection: Optional[Any] = None,
     ) -> DataFrame:
-        """Actually execute the query, and return a DataFrame. Unlike :meth:`.execute`, this method will always keep the results
-        as a DataFrame, even if the finalizer implies a singleton results (e.g. ONE, ONE_OR_NONE, or FIRST).
+        """Actually execute the query, and return a DataFrame. Unlike :meth:`.execute`, this method will always keep the
+        results as a DataFrame, even if the finalizer implies a singleton results (e.g. ONE, ONE_OR_NONE, or FIRST).
 
         Parameters
         ----------
@@ -154,8 +155,10 @@ class FinalizedChalkQuery:
                 (
                     "This query specified an incremental configuration, which has not been applied. "
                     "This is likely because the resolver is being executed directly. "
-                    "The query will be attempted without any high-water-mark timestamp. This will attempt to select all data, "
-                    "or if the filters depend on the incremental timestamp, will result in a query execution error. "
+                    "The query will be attempted without any high-water-mark timestamp. "
+                    "This will attempt to select all data, "
+                    "or if the filters depend on the incremental timestamp, "
+                    "will result in a query execution error. "
                 )
             )
 
@@ -184,6 +187,58 @@ class FinalizedChalkQuery:
 
         return DataFrame(df)
 
+    def execute_to_dataframe_batches(
+        self,
+        expected_features: Optional[Sequence[Feature]] = None,
+        connection: Optional[Any] = None,
+    ) -> Iterator[DataFrame]:
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
+        if self.incremental_settings is not None:
+            # FIXME: Move the incrementalization logic here, so then the `execute` and `execute_to_dataframe`
+            # methods can take the hwm timestamp as a paramater, to allow for direct execution
+            warnings.warn(
+                (
+                    "This query specified an incremental configuration, which has not been applied. "
+                    "This is likely because the resolver is being executed directly. "
+                    "The query will be attempted without any high-water-mark timestamp. "
+                    "This will attempt to select all data, "
+                    "or if the filters depend on the incremental timestamp, "
+                    "will result in a query execution error. "
+                )
+            )
+
+        def converter_getters(column_names: List[str]) -> Dict[str, FeatureConverter]:
+            # First map the column names to determine the feature fqns
+            col_name_mapping = self.get_col_name_mapping(column_names, expected_features)
+            return {k: Feature.from_root_fqn(v).converter for (k, v) in col_name_mapping.items()}
+
+        pa_tables = self.source.execute_query_to_batches(self, converter_getters, connection)
+        i = 0
+        for pa_table in pa_tables:
+            i += 1
+            chalk_logger.info(f"Processing batch {i} from query with shape {pa_table.shape}.")
+            col_name_mapping = self.get_col_name_mapping(pa_table.column_names, expected_features)
+            pa_table = pa_table.select(col_name_mapping.keys())
+            pa_table = pa_table.rename_columns([col_name_mapping[x] for x in pa_table.column_names])
+            df = pl.from_arrow(pa_table)
+            assert isinstance(df, pl.DataFrame)
+
+            if self._finalizer == Finalizer.ONE:
+                if len(df) != 1:
+                    raise ValueError(f"Expected exactly one row; got {len(df)} rows")
+
+            if self._finalizer == Finalizer.ONE_OR_NONE:
+                if len(df) > 1:
+                    raise ValueError(f"Expected zero or one rows; got {len(df)} rows")
+
+            if self._finalizer in (Finalizer.ONE, Finalizer.ONE_OR_NONE, Finalizer.FIRST):
+                df = df.slice(0, 1)
+
+            yield DataFrame(df)
+
     def get_col_name_mapping(
         self,
         result_columns: List[str],
@@ -209,7 +264,7 @@ class FinalizedChalkQuery:
         """
         ans: Dict[str, str] = {}
         normalized_to_original_col = {normalize_string_for_matching(x): x for x in result_columns}
-        for (k, v) in self._fields.items():
+        for k, v in self._fields.items():
             original_col_name = normalized_to_original_col.get(normalize_string_for_matching(k))
             if original_col_name is None:
                 raise ValueError(f"Column {k} was not returned by the query.")
