@@ -24,6 +24,14 @@ class KafkaSource(StreamSource, SinkIntegrationProtocol, BaseModel, frozen=True)
     files.
     """
 
+    ssl_ca_file: Optional[str] = None,
+    """
+    An S3 or GCS URI that points to the certificate authority file that should be
+    used to verify broker certificates. You must configure the appropriate AWS or
+    GCP integration in order for Chalk to be able to access these files.
+    """
+
+
     client_id_prefix: str = "chalk/"
     group_id_prefix: str = "chalk/"
 
@@ -68,6 +76,7 @@ class KafkaSource(StreamSource, SinkIntegrationProtocol, BaseModel, frozen=True)
         bootstrap_server: Optional[Union[str, List[str]]] = None,
         topic: Optional[Union[str, List[str]]] = None,
         ssl_keystore_location: Optional[str] = None,
+        ssl_ca_file: Optional[str] = None,
         client_id_prefix: Optional[str] = None,
         group_id_prefix: Optional[str] = None,
         security_protocol: Optional[str] = None,
@@ -102,6 +111,7 @@ class KafkaSource(StreamSource, SinkIntegrationProtocol, BaseModel, frozen=True)
             sasl_password=sasl_password or load_integration_variable(name="KAFKA_SASL_PASSWORD", integration_name=name),
             name=name,
             late_arrival_deadline=late_arrival_deadline,
+            ssl_ca_file=ssl_ca_file or load_integration_variable(name="KAFKA_SSL_CA_FILE", integration_name=name),
         )
 
     def _config_to_json(self) -> Any:
