@@ -1,7 +1,7 @@
 import collections.abc
 from typing import Any, Dict, Iterable, List, Optional, Sequence, Set, Tuple, TypeVar, Union, cast, overload
 
-from typing_extensions import get_args, get_origin
+from typing_extensions import Annotated, get_args, get_origin
 
 try:
     from types import UnionType
@@ -9,13 +9,23 @@ except ImportError:
     UnionType = Union
 
 _NoneType = type(None)
+_AnnotatedType = type(Annotated[int, ""])
 
 T = TypeVar("T")
 
 
-def unwrap_optional_if_needed(typ: Any):
+def unwrap_annotated_if_needed(typ: Any):
+    args = get_args(typ)
+    if type(typ) is _AnnotatedType and hasattr(typ, "__metadata__"):
+        return args[0]
+    return typ
+
+
+def unwrap_optional_and_annotated_if_needed(typ: Any):
     origin = get_origin(typ)
     args = get_args(typ)
+    typ = unwrap_annotated_if_needed(typ)
+
     if origin in (Union, UnionType) and len(args) == 2 and any(d == _NoneType for d in args):
         return next(m for m in get_args(typ) if m != _NoneType)
 
