@@ -17,6 +17,14 @@ def removeprefix(text: str, prefix: str):
     return text[len(prefix) :] if text.startswith(prefix) else text
 
 
+def removesuffix(text: str, suffix: str):
+    """
+    This functionality is available in Python 3.9 as text.removesuffix(suffix),
+    but we're supporting down to 3.8, where it is not available.
+    """
+    return text[: -len(suffix)] if text.endswith(suffix) else text
+
+
 def comma_whitespace_split(value: str):
     return re.split(r"\s*,\s*", value)
 
