@@ -20,6 +20,7 @@ except ImportError:
 class ParsedAnnotation(NamedTuple):
     annotation: str
     include_in_protocol_cls: bool
+    protocol_annotation: str
 
 
 class ParsedFeaturesClass(NamedTuple):
@@ -123,12 +124,18 @@ class StubGenerator:
                 annotations[f.attribute_name] = ParsedAnnotation(
                     "DataFrame",
                     include_in_protocol_cls=True,
+                    protocol_annotation="DataFrame",
                 )
             else:
                 # Make a best-effort to figure out imports.
+                annotation = self._register_imports_for_annotation(f.typ.parsed_annotation)
                 annotations[f.attribute_name] = ParsedAnnotation(
-                    self._register_imports_for_annotation(f.typ.parsed_annotation),
+                    annotation,
                     include_in_protocol_cls=not f.is_windowed_pseudofeature and not f.is_autogenerated,
+                    # If it's a features class (i.e. a has-one), not specifying
+                    # the type in the protocol class so the type checker can
+                    # still match feature classes that have a circular reference
+                    protocol_annotation="..." if f.typ.is_features_cls else annotation,
                 )
         if features_cls.__name__ in self._parsed_feature_classes:
             raise ValueError(
@@ -149,7 +156,7 @@ class StubGenerator:
         self._add_import("chalk.utils.duration.Duration", "Duration")
         self._add_import("chalk.features.Tags", "Tags")
         lines = [
-            "# AUTO-GENERATED FILE. Do not edit. Run chalkpy stubgen to generate.",
+            "# AUTO-GENERATED FILE. Do not edit. Run `chalk stubgen` to generate.",
             "# fmt: off",
             "# isort: skip_file",
             "from __future__ import annotations",
@@ -167,7 +174,7 @@ class StubGenerator:
         #
         # In no particular order:
         #
-        # Class 1: The metaclasss. The metaclass includes @property descriptors for each class variable.
+        # Class 1: The metaclass. The metaclass includes @property descriptors for each class variable.
         # These descriptors are used to annotation class attributes as Type[feature]
         # Pyright requires that all annotations are types, not instances.
         #
@@ -247,7 +254,7 @@ class {feature_cls_name}Protocol(Protocol):"""
                 has_proto_annotation = True
                 lines.append(
                     f"""\
-    {attribute}: {annotation.annotation}"""
+    {attribute}: {annotation.protocol_annotation}"""
                 )
             if not has_proto_annotation:
                 lines.append(
